#!/usr/bin/env python3

# from scapy.all import send, IP, ICMP
from scapy.all import *
# from scapy.contrib import roce
import random
import sys
import struct
import time
import random
import binascii
import ipaddress


# dstMAC = "56:2B:95:DB:33:39"
# dstMAC = "b8:ce:f6:61:a0:f6"
# dstMAC = "ff:ff:ff:ff:ff:ff"
# dstMAC = "b8:ce:f6:61:9f:96" # host

# dstMAC = "b8:ce:f6:61:9f:96" # host13
dstMAC = "b8:ce:f6:61:9f:9a"   # dpu13

srcIP = "11.11.11.1"
dstIP = "10.1.0.3"

# rocev2_port = 4791 # Default RoCEv2=4791
rocev2_port = 5000

# PSN 在所有数据包的基本传输报头 (BTH) 中传输。它们用于检测丢失或无序的数据包，并且为了可靠的服务，
# 将响应包与给定的请求包相关联。每个 IBA QP 由一个发送队列和一个接收队列组成；同样，EE 上下文有一个发送端和一个接收端。
# 请求者发送队列上的 PSN 与响应者相应接收队列上的 PSN 之间存在关系。因此，QP（或 EE 上下文）的每一半维护一个独立的PSN；
# 在给定队列对的发送队列和接收队列上所使用的 PSN，或者在不同的连接之间所使用的 PSN 是没有关系的。


class BTH(Packet):
	''' RDMA 的 BTH (Base Transport Header) 报头的字段定义 '''

	name = "BTH"
	fields_desc = [
		ByteField("opcode", 0),
		BitField("solicitedEvent", 0, 1),
		BitField("migReq", 0, 1),
		BitField("padCount", 0, 2),
		BitField("transportHeaderVersion", 0, 4),
		XShortField("partitionKey", 0),
		XByteField("reserved1", 0),
		ThreeBytesField("destinationQP", 0),
		BitField("ackRequest", 0, 1),
		BitField("reserved2", 0, 7),
		ThreeBytesField("packetSequenceNumber", 0)
	]


class RETH(Packet):
	''' RDMA 的 RETH (RDMA Extended Transport Header) 报头的字段定义, 
	    它包含 RDMA 操作的附加传输字段。RETH 只存在于 RDMA 请求的第一个 (或唯一) 数据包中 '''

    # Virtual Address 表示缓冲区的起始地址
	name = "RETH"
	fields_desc = [
		BitField("virtualAddress", 0, 64),
		IntField("rKey", 0),
		IntField("dmaLength", 0)
	]


class iCRC(Packet):
	''' RDMA 的 ICRC (不变CRC) 报头的字段定义 (它覆盖了从源到目的地的数据包中不发生变化的字段) '''

	name = "iCRC"
	fields_desc = [
		IntField("iCRC", 0),
		
	]

# PSN 表示 RoCEv2 报文的序列号，可通过检测 PSN 是否连续来判断是否存在丢失的数据包，若出现了丢包，就会返回NAK报文
packetSequenceNumber = 0

def makeRocev2Write(payload=0xdeadbeef, address=0x0):
	''' 制作具有 32 比特大小有效载荷的 RDMA Write 数据包 '''

	global packetSequenceNumber
	# 在 InfiniBand 框架中，需要在特定节点之间提供隔离，并创建只有这些节点能够通信的 "Virtual Fabric"
	# 我们称之为分区(Partitioning)。而分区我们使用分区密钥 (P_Key) 来表示和实现，这是一个 16 位值，由两部分组成：
	# The msb is the membership bit:
		# 0: limited membership
		# 1: full membership
	# The lower 15 bits are the key portion which defines the partition value
	# 只有配置为同一分区的 QP 才能进行通信，而且其中至少有一个 QP 应是该分区的正式成员
	partitionKey = 0
	# Destination Queue Pair，目的QP，用来标识一条RoCEv2流，相当于RoCEv2报文中的目的端口Dport (Destination port)
	destinationQP = 0
	# RETH 报头中的 dmaLength 字段表示 DMA 操作的长度 (以字节为单位)
	dmaLength = 32
	virtualAddress = address   # RDMA 缓冲区的起始虚拟内存地址
	rKey = 0                   # MR 被注册的远程键, 用于授权其他主机访问本地主机已注册的指定内存区域
	
	
	iCRC_checksum = 0 #TODO: calculate this? Or ignore?
	
	payload = struct.pack(">I", payload)
	# virtualAddress = struct.pack(">Q", virtualAddress)
	
	packetSequenceNumber = packetSequenceNumber + 1

	pkt = Ether(src="b8:ce:f6:61:a0:f2", dst=dstMAC)
	pkt = pkt / IP(src=srcIP,dst=dstIP,ihl=5,flags=0b010,proto=0x11)
	pkt = pkt / UDP(sport=0xc0de,dport=rocev2_port,chksum=0)
	pkt = pkt / BTH(opcode=0b01010,partitionKey=partitionKey,destinationQP=destinationQP, packetSequenceNumber=packetSequenceNumber) # WRITE-ONLY 类型的 RDMA 数据包
	pkt = pkt / RETH(dmaLength=dmaLength,virtualAddress=virtualAddress,rKey=rKey)
	pkt = pkt / Raw(payload)
	pkt = pkt / iCRC(iCRC=iCRC_checksum)
	
	return pkt


def calc_iCRC(pkt):
	''' 计算 iCRC 校验和 '''

	#pkt_icrc_mod = pkt.copy()
	
	print("Calculating iCRC on packet", pkt)
	
	print("version", pkt["IP"].version)
	print("ihl", pkt["IP"].ihl)
	
	#CRC part 1
	crc_part_1 = struct.pack("!Q", 0xffffffffffffffff)
	
	#CRC part 2
	tmp1 = (pkt["IP"].version<<4) + (pkt["IP"].ihl)
	print("tmp1: %i(0x%x)" %(tmp1, tmp1) )
	tmp2 = (pkt["IP"].flags<<3) + (pkt["IP"].frag)
	print("tmp2: %i(0x%x)" %(tmp2, tmp2) )
	print(pkt["IP"].len)
	crc_part_2 = struct.pack("!BBHH", tmp1, 0xff, pkt["IP"].len, tmp2 )
	
	#CRC part 3
	srcIP = int(ipaddress.ip_address(pkt["IP"].src))
	print("srcIP", srcIP)
	crc_part_3 = struct.pack("!BBHI", 0xff, pkt["IP"].proto, 0xffff, srcIP )
	
	#CRC part 4
	dstIP = int(ipaddress.ip_address(pkt["IP"].dst))
	print("dstIP", dstIP)
	crc_part_4 = struct.pack("!IHH", dstIP, pkt["UDP"].sport, pkt["UDP"].dport )
	
	#CRC part 5
	tmp3 = (pkt["BTH"].solicitedEvent<<4+2+1) + (pkt["BTH"].migReq<<4+2) + (pkt["BTH"].padCount<<4) + (pkt["BTH"].transportHeaderVersion)
	print(pkt["UDP"].len, 0xffff, pkt["BTH"].opcode, tmp3, pkt["BTH"].partitionKey)
	crc_part_5 = struct.pack("!HHBBH", pkt["UDP"].len, 0xffff, pkt["BTH"].opcode, tmp3, pkt["BTH"].partitionKey )
	
	#CRC part 6
	dqp_1_1B = pkt["BTH"].destinationQP>>16
	dqp_2_2B = pkt["BTH"].destinationQP&0xffff
	tmp4 = (pkt["BTH"].ackRequest<<7) + (pkt["BTH"].reserved2)
	psn_1_1B = pkt["BTH"].packetSequenceNumber>>16
	psn_2_2B = pkt["BTH"].packetSequenceNumber&0xffff
	crc_part_6 = struct.pack("!BBHBBH", 0xff, dqp_1_1B, dqp_2_2B, tmp4, psn_1_1B, psn_2_2B)
	print("crc_part_1", crc_part_1)
	print("crc_part_2", crc_part_2)
	print("crc_part_3", crc_part_3)
	print("crc_part_4", crc_part_4)
	print("crc_part_5", crc_part_5)
	print("crc_part_6", crc_part_6)
	
	
	crc_indata_full = crc_part_1+crc_part_2+crc_part_3+crc_part_4+crc_part_5+crc_part_6
	
	print("crc_indata_full", crc_indata_full)
	
	output = binascii.crc32( crc_indata_full )
	print("iCRC checksum: ", output)
	return output


def makeRocev2Send():
	''' 制作 RDMA SEND 数据包 '''

	global packetSequenceNumber
	partitionKey = 0
	destinationQP = 0
	rKey = 0x42069     # MR 被注册的远程键, 用于授权其他主机访问本地主机已注册的指定内存区域
	
	
	packetSequenceNumber = packetSequenceNumber + 1

	
	pkt = Ether(src="b8:ce:f6:61:a0:f2", dst=dstMAC)
	pkt = pkt / IP(src=srcIP, dst=dstIP, ihl=5, flags=0b010, proto=0x11)
	pkt = pkt / UDP(sport=0xc0de,dport=rocev2_port,chksum=0)
	# BTH 的操作码字段: WRITE-ONLY: 0b01010, SEND-ONLY: 0b00100
	pkt = pkt / BTH(opcode=0b00100,partitionKey=partitionKey,destinationQP=destinationQP, packetSequenceNumber=packetSequenceNumber) 
	
	
	#Force some field updates...
	pkt["IP"].len = 44    # 48?
	pkt["UDP"].len = 24
	
	iCRC_checksum = calc_iCRC(pkt)
	
	pkt = pkt / iCRC(iCRC=iCRC_checksum)
	
	pkt.show2()
	pkt.show2()
	
	return pkt


def makeIPPacket():
	''' 制作 IP 数据包 '''

	pkt = Ether(src="b8:ce:f6:61:a0:f2",dst=dstMAC)
	pkt = pkt / IP(src=srcIP,dst=dstIP)
	return pkt


def makeUDPPacket():
	''' 制作 UDP 数据包 '''

	pkt = Ether(src="b8:ce:f6:61:a0:f2",dst=dstMAC)
	pkt = pkt / IP(src=srcIP,dst=dstIP) / UDP()
	return pkt




pkt = makeRocev2Send()
print("Sending packet", pkt)
sendp(pkt, iface="enp4s0f0")
wrpcap("rocev2_send_pkt.pcap", pkt)


'''
numFlows = 5
flowHashes = []
for flowID in range(numFlows):
	flowHash = random.randint(0,2**64-1)
	flowHashes.append(flowHash)
	
#Send traffic
flowID = 0
while True:
	flowID += 1
	if flowID >= numFlows:
		flowID = 0
	
	payload = flowID
	address = flowHashes[flowID]
	
	print("Transmitting telemetry data. Flow:%i, payload:%i, hash:%i" %(flowID, payload, address))
	
	pkt = makeRocev2Write(payload=payload, address=address)
	print("Sending packet", pkt)
	sendp(pkt, iface="enp4s0f0")
	
	time.sleep(0.5)
'''
